#region StandardUsing
using System;
using QPlatform.CoreBase;
using QPlatform.HMIProject;
using UAManagedCore;
using OpcUa = UAManagedCore.OpcUa;
using QPlatform.NetLogic;
using QPlatform.OPCUAServer;
using QPlatform.UI;
#endregion

public class CreateUserPanelLogic : QPlatform.NetLogic.BaseNetLogic
{
    [ExportMethod]
    public void CreateUser(string username, string password, string locale, out NodeId result)
    {
		result = NodeId.Empty;
		if (String.IsNullOrEmpty(username))
		{
			Log.Error("Cannot create user with empty username");
			return;
		}

		result = GenerateUser(username, password, locale);
    }

    private NodeId GenerateUser(string username, string password, string locale)
    {
        var user = InformationModel.MakeObject<QPlatform.Core.User>(username);
        user.Password = password;
        user.LocaleIds = new string[] { locale };

		var users = GetUsers();
		if (users == null)
		{
			Log.Error("Unable to get users");
			return NodeId.Empty;
		}
		users.Add(user);

		return user.NodeId;
	}

    IUANode GetUsers()
	{
		var resolvedNode = LogicObject.Context.ResolvePath(LogicObject, "{Users}@Jump");
		if (resolvedNode == null)
			return null;
		if (resolvedNode.Nodes.Count == 0)
			return null;

		return resolvedNode.Nodes[0];
	}
}
